@@ -9,11 +9,17 @@ from alteia.core.errors import ResponseError
 from alteia.core.resources.resource import Resource, ResourcesWithTotal
 from alteia.core.utils.typing import ResourceId, SomeResourceIds
 from alteia.sdk import SDK
-from PyInquirer import Separator, Token, prompt, style_from_dict
-from tabulate import tabulate
 
+from tabulate import tabulate
 from alteia_cli import AppDesc, utils
 from alteia_cli.sdk import alteia_sdk
+# Hack to circumvent ImportError starting from Python 3.10
+# PyInquirer is not maintained anymore and uses a not up-to-date version
+# of prompt-toolkit.
+import collections.abc
+setattr(collections, 'Iterable', collections.abc.Iterable)
+setattr(collections, 'Mapping', collections.abc.Mapping)
+from PyInquirer import Separator, Token, prompt, style_from_dict  # noqa: E402
 
 app = typer.Typer()
 app_desc = AppDesc(app, name='analytics', help='Interact with analytics.')
@@ -261,14 +267,18 @@ ANALYTIC_DESC_SCHEMA = _get_analytic_schema('share/analytic_schema.yaml')
 
 @app.command(name='list')
 def list_analytics(
-    limit: int = typer.Option(default=100, help='Max number of analytics returned.'),
+    limit: int = typer.Option(
+        100, '--limit', '-n', min=1,
+        help='Max number of analytics returned.'),
     display_all: bool = typer.Option(
         False, '--all', case_sensitive=False,
         help='If set, display all kinds of analytics '
              '(otherwise only custom analytics are displayed).'
     ),
 ):
-    """ List the analytics. """
+    """
+        List the analytics.
+    """
     sdk = alteia_sdk()
     with utils.spinner():
         search_filter = {'is_backoffice': {'$ne': True}}
@@ -296,7 +306,7 @@ def list_analytics(
             'Version': [r.version if hasattr(r, 'version') else '' for r in results],
             'Identifier': [r.id for r in results]
         }
-        print(tabulate(
+        typer.secho(tabulate(
             table,
             headers='keys',
             tablefmt='pretty',
@@ -322,7 +332,9 @@ def create(
         help='Path of the Analytic description (YAML file).'),
     company: str = typer.Option(default=None, help='Company identifier.'),
 ):
-    """ Create a new analytic. """
+    """
+        Create a new analytic.
+    """
     analytic_desc = parse_analytic_yaml(description)
     analytic = Analytic.from_yaml(analytic_desc)
     typer.secho('✓ Analytic description is valid', fg=typer.colors.GREEN)
@@ -452,7 +464,7 @@ def parse_analytic_yaml(analytic_yaml_path: Path) -> Dict:
             'Found error(s):',
             fg=typer.colors.RED
         )
-        print(errors)
+        typer.secho(errors)
         raise typer.Exit(2)
     else:
         return analytic
@@ -484,11 +496,32 @@ def find_analytics(sdk, analytic_name: str, version_range: str = None,
     return results
 
 
-def get_companies_hierarchy(sdk: SDK):
+def get_companies_hierarchy(sdk: SDK, permission_name: str):
     user = sdk.users.describe()
-    company_ids = cast(SomeResourceIds, [c['id'] for c in user.companies])
+    company_ids = cast(SomeResourceIds,
+                       get_companies_where_permission(sdk, permission_name))
+
     user_companies = [c.__dict__ for c in cast(List[Resource],
                                                sdk.companies.describe(company_ids))]
+    # The user does not necessarily have the permission to describe
+    # companies available for sharing. In this case, we can only show the id
+    # of the company.
+    root_company_id = get_root_company(sdk)
+    for cid in company_ids:
+        if not any(c['_id'] == cid for c in user_companies):
+            if cid == root_company_id:
+                company = {
+                    '_id': cid,
+                    'name': 'Root company'
+                }
+            else:
+                company = {
+                    '_id': cid,
+                    'name': cid,
+                    'root_company': root_company_id
+                }
+            user_companies.append(company)
+
     companies_by_id = {c['_id']: c for c in user_companies}
     hierarchised_companies = list()
     # Build a hierarchy of company (company object with property children)
@@ -513,24 +546,56 @@ def get_companies_hierarchy(sdk: SDK):
     return (user_companies, hierarchised_companies, user)
 
 
-def get_root_company(sdk: SDK) -> ResourceId:
-    # Try to find a root company available to the user
+def get_companies_where_permission(sdk: SDK, permission_name) -> List[ResourceId]:
+    def filter_have_scope_where_permission(have_scope):
+        if have_scope is None:
+            return
+        for has_scope in have_scope:
+            scope = has_scope.get('scope')
+            if scope is None:
+                continue
+            service_perms = scope.get('analytics-service')
+            if service_perms is None:
+                continue
+            for p in service_perms:
+                if p == permission_name:
+                    yield has_scope
+
+    token = sdk._connection._token_manager.describe_token()
+    user_in_token = token['user']
+    companies = set()
+    for has_scope in filter_have_scope_where_permission(
+            user_in_token.get('permissions')):
+        companies.update(has_scope['companies'])
+    for has_scope in filter_have_scope_where_permission(
+            user_in_token.get('companies')):
+        companies.add(has_scope['id'])
+
+    return list(companies)  # convert to list to be JSON serializable
+
+
+def get_root_company(sdk: SDK, permission_name: str = None) -> ResourceId:
+    # Assuming a user can only belong to one domain
     user = sdk.users.describe()
-    company_ids = cast(SomeResourceIds, [c['id'] for c in user.companies])
-    user_companies = cast(List[Resource], sdk.companies.describe(company_ids))
-    root_company = None
-    for c in user_companies:
-        if c.domain == user.domain and not getattr(c, 'root_company', None):
-            root_company = c
-            break
-    else:
-        typer.secho(
-            '✖ No sufficient permissions to access the root company '
-            'of your domain.',
-            fg=typer.colors.RED
-        )
-        raise typer.Exit(2)
-    return root_company._id
+    first_company_id = cast(ResourceId, user.companies[0]['id'])
+    first_company = cast(Resource, sdk.companies.describe(first_company_id))
+    root_company_id = getattr(first_company, 'root_company', None)
+    if root_company_id is None:
+        root_company_id = first_company._id
+
+    # Ensure the user has the given permission
+    # in the context of the root company
+    if permission_name is not None:
+        user_company_ids = get_companies_where_permission(sdk, permission_name)
+        if root_company_id not in user_company_ids:
+            typer.secho(
+                '✖ No sufficient permissions to access the root company '
+                'of your domain.',
+                fg=typer.colors.RED
+            )
+            raise typer.Exit(2)
+
+    return root_company_id
 
 
 @app.command()
@@ -565,13 +630,16 @@ of your domain, the analytic will still be shared with that company.'''
             )
             raise typer.Exit(2)
 
+    permission_name = 'unshare-analytic-with-company:external'
+
     def company_selection(analytic_current_companies):
         def validate_answer(answer):
             if len(answer):
                 return True
             return 'You must choose at least company.'
 
-        companies, hierarchised_companies, _ = get_companies_hierarchy(sdk)
+        companies, hierarchised_companies, _ = \
+            get_companies_hierarchy(sdk, permission_name)
         questions = [
             {
                 'type': 'checkbox',
@@ -628,7 +696,7 @@ of your domain, the analytic will still be shared with that company.'''
     if company:
         companies = [company]
     elif domain:
-        companies = [get_root_company(sdk)]
+        companies = [get_root_company(sdk, permission_name)]
     else:
         companies = company_selection(analytics)
         if companies is None:
@@ -693,13 +761,16 @@ companies of your domain and is equivalent to using the
     """ Share an analytic """
     sdk = alteia_sdk()
 
+    permission_name = 'share-analytic-with-company:external'
+
     def company_selection(analytics):
         def validate_answer(answer):
             if len(answer):
                 return True
             return 'You must choose at least one company.'
 
-        companies, hierarchised_companies, _ = get_companies_hierarchy(sdk)
+        companies, hierarchised_companies, _ = \
+            get_companies_hierarchy(sdk, permission_name)
         questions = [
             {
                 'type': 'checkbox',
@@ -768,7 +839,7 @@ companies of your domain and is equivalent to using the
     if company:
         companies = [company]
     elif domain:
-        companies = [get_root_company(sdk)]
+        companies = [get_root_company(sdk, permission_name)]
     else:
         companies = company_selection(analytics)
         if companies is None:
@@ -815,7 +886,9 @@ def delete(analytic_name: str = typer.Argument(...),
                                        help='Version range of the analytic '
                                        'in SemVer format. If not provided, '
                                        'all the versions will be deleted.')):
-    """ Delete an analytic. """
+    """
+        Delete an analytic.
+    """
     sdk = alteia_sdk()
     analytics = find_analytics(sdk, analytic_name=analytic_name,
                                version_range=version)
